                   CREATING DOCUMENTS WITH VI

The "browse.v" file explained how to read through a file using
"vi".  Unless the text suggests a different way to move through
the file, just read through this file by pressing Enter or
Ctrl-D.

***************************************************************
*******              Part I: General Tips               *******
***************************************************************

The wrapmargin Option:

When you are typing lines of text in paragraph form, your job
is easier if you don't have to remember to press Enter when you
reach the end of a line.  Try entering the following command.

:set wrapmargin=8

Now go to the next blank line, type "a" to begin appending text,
and start typing the contents of this paragraph, beginning
with "When you".  Don't press Enter when you start copying the
second line.  Keep on going.  Press Esc to leave Insert Mode
after a few lines.

You should see that the word "easier" automatically jumps down
to a new line.  The option "wrapmargin=8" says that if you end a
word within eight spaces of the right margin of the screen, the
next word should wrap around to begin on the next line.  You do
not have to press Enter.  With this kind of wrap-around margin,
you can just keep on typing words.  The only time you need to
press Enter will probably be at the end of paragraphs.


Abbreviations:

"vi" lets you define abbreviations for commonly used words and
phrases.  This uses the ":ab" (abbreviate) command.  As an
example, try typing

:ab xyz XYZ Widgets Inc. International

and then press Enter.  This says that "xyz" will be an
abbreviation for "XYZ Widgets Inc. International".  To see how
it works, move the cursor to the next blank line, press "a" to
begin appending text, and type

xyz welcomes you!

You will see that as soon as you type the space after "xyz", the
abbreviation is expanded into the associated phrase.  Now move
the cursor to the next blank line, press "a" to begin appending
text, and type

This is the XYZ factory.

This time, you'll see that "XYZ" is not expanded into the
abbreviation.  The abbreviation feature pays attention to the
case of letters.  Now move the cursor to the next blank line,
press "a" to begin appending text, and type

xyzwill not expand.

When you type this, you will see that the "xyz" is not expanded
because it is part of another word.  "vi" only expands
abbreviations when they are not part of other words.

If you want to get rid of an abbreviation that has been set, use
the ":una" (unabbreviate) command.  For example, type

:una xyz

This tells "vi" to forget about the "xyz" abbreviation.


Command Files for "vi":

An abbreviation only lasts for the duration of a "vi" session.
If you set up an abbreviation now, it will be remembered as long
as you stay in "vi", but will be forgotten when you leave "vi".
This means that you have to set up common abbreviations every
time you start "vi".  This would be a lot of work if you did it
by hand, but you can simplify things quite a bit by setting up a
command file.

A command file contains several commands that can be executed as
if they were typed in a "vi" session.  For example, you might
use "vi" to create a file with the contents

set wrapmargin=8
set tabstop=5
ab mks Mortice Kern Systems Inc.
ab aa another abbreviation
ab bb another abbreviation
   ...and so on...

setting all the options you want to use and all the
abbreviations you commonly need.  The file can only contain
instructions that normally start with a colon (:) in "vi", and
you omit the colons in the command file.  You can execute all
the instructions in the command file inside "vi" with the
instruction

:so filename

where filename is the name of your command file.  "so" stands
for source, and it tells "vi" that the given file should be
taken as the source of several commands.

As a matter of fact, you can execute the commands in a command
file when you first start "vi".  You do this by starting "vi"
with the command line

vi -c "so cmdfile" editfile

where cmdfile is the name of your command file and editfile is
the name of the file you want to edit.  After I've discussed
aliases in Chapter 7, you might want to set up a KornShell alias
for

vi -c "so cmdfile"


***************************************************************
*******            Part II: Combining Files             *******
***************************************************************

Occasionally, you may want to combine several files into a
single document.  For example, you may have a table of data
stored in one file and want to add the table to another file.
To demonstrate how this can be done, move the cursor to the next
blank line and type

:r sonnet

You should see the contents of the file read in after the line
that holds the cursor.  "r" stands for read, and it reads in the
text of a file.  This text is added to the current file after
the line indicated by the cursor.

The same sort of command can be used to combine the chapters of
a document into a single file.  For example, consider the
following commands (but don't enter them):

:r chapter1
G
:r chapter2
G
:r chapter3

These will read in chapters that are stored in separate files.
Notice that you had to add "G" commands to go to the end of the
file after each "r" operation so that the next input file would
be added to the end of the text.

It's a good idea to store large documents in several smaller
files and only combine them when you are ready to create the
finished product.  The smaller the file, the faster "vi" can
read and write the text.  Small files are also more convenient
for you to edit, because it takes less time to find the text you
want to change.


***************************************************************
*******             Part III: Sorting Data              *******
***************************************************************

In many types of reports, it is convenient to be able to sort a
few lines of text (for a table of information).  As an example,
consider the following lines.

a  -- append after cursor
o  -- open an input line
dd -- delete a line
cc -- change a line
w  -- move forward a word
b  -- move backward a word
x  -- delete a letter
u  -- undo

I entered these descriptions of "vi" commands in the order I
thought of them, but it would be nice to sort them into
alphabetical order.  To do this, you can send the preceding
lines through the "sort" command.

First, you have to find out the line numbers of the lines you
want to sort.  To do this, move the cursor to the first line of
the preceding table and press Ctrl-G.  "vi" displays the name of
the file and a notation of the form "line N of M", where "N" is
the line number of the current line and "M" is the total number
of lines in the file.  Note the line number (it should be 172,
or a number close to that).  Move the cursor to the last line of
the table and press Ctrl-G again.  You should find out the line
number is 179.  Now type

:172,179!sort

and press Enter.  This asks "vi" to send lines 172 to 179
through the "sort" command.  The lines will be replaced with the
output of the sort command, namely the sorted lines.  Go back
and check the list, and you will see that it is now sorted.

The same technique can be used to send lines through other
commands if you like.  However, "sort" is probably used in this
context more often than any other command.

If you do not specify a range of lines for a command, the
command will just be executed.  Its output will be displayed on
the screen, but it will not replace any of the text of the file.
For example, write out this file and then try the command

:!wc doc.v

Because "wc" prints the number of lines, words, and characters
in a text file, this command gives you a quick word count for
your document.


***************************************************************
*******           Part IV: Formatting by Hand           *******
***************************************************************

Most UNIX systems do not have word processors; the most common
way to create documents on UNIX is to use a text formatting
program named "troff", discussed in Chapter 10.  Using "troff"
is definitely recommended for long documents, but for short
documents like letters, you can often format text "by hand",
using "vi" to create a text file that can be printed directly on
a printer.


Page Breaks:

The Ctrl-L character is called the formfeed.  You can use it to
separate pages.  However, there's a catch.  On a UNIX system,
you can't just append a Ctrl-L character to your text, since
Ctrl-L is a special instruction that works even when you're in
Insert Mode.  This instruction tells "vi" to redraw the screen;
it's needed because there are a variety of ways in which the
text that is displayed on your screen can get messed up by
output written by other processes.  (It's too complicated to
explain how this might happen, but take my word for it, once in
a while you have to tell "vi" that some other program messed up
your screen and you want "vi" to clean things up again.)

However, you can still add a Ctrl-L character to your text; you
just have to be a little more clever about it.  To see what you
have to do, move the cursor to the next blank line and press "a"
to begin appending text.  First press Ctrl-V, then press Ctrl-L.
The Ctrl-V tells "vi" to append the next character you enter,
even if that character usually has a special meaning.  After you
do this, you should see that the screen displays "^L", standing
for Ctrl-L. By inserting these formfeed characters, you can
break a document into pages.

On a normal line printer, an 8 1/2- by 11-inch sheet of paper
holds a maximum of 66 lines.  For the purpose of formatting, 54
lines fit nicely on a page, with an inch of space at the top and
the bottom for margins.  Thus you should consider inserting
formfeeds every 54 lines (give or take one or two).

If you indent a paragraph with a tab character, the amount of
indentation is determined by the tab stops set for the printer
on which the document is printed.  If you indent a paragraph
with spaces, the amount of indentation does not depend on the
printer setting.


Indentation:

The ">" command can indent several lines in a uniform way.  For
example, try typing

:1,274>

and press Enter.  This indents the first 274 lines of this file.
You can get rid of the indentation by typing

:1,274<

and pressing Enter.  As an instruction to try later, you can
indent the entire file with

:1,$>

When you are giving a range of lines to a command, "$" is a
special symbol that stands for the last line of the file.  The
reason I suggest you try this later is that the cursor will move
to the last indented line (the end of the file), which would
cause you to lose your place in this file.

After you have formatted a document, write it out to a file
(with ":w").  On a true UNIX system, you could then print the
file using the "lp" or "lpr" commands.  On DOS, you can quit the
MKS Tools and use "PRINT".
