                   PROGRAMMING TIPS USING VI

The "browse.v" file explained how to read through a file using
"vi".  Unless the text suggests a different way to move through
the file, just read through this file by pressing Enter or
Ctrl-D.

***************************************************************
*******           Part I: Indentation Control           *******
***************************************************************

The source code for a program differs from ordinary text in
several ways.  One of the most important of these is the way
source code uses indentation.  Indentation shows the logical
structure of the program: the way statements are grouped into
blocks.


The autoindent Option:

Issue the command

:set autoindent

(Don't forget to press Enter after you have typed this.)  The
command turns on a "vi" option that controls indentation when
you enter source code.  To see how it works, copy (append) the
next two lines into the blank line that follows them.

    if (a > b) max = a;
        else max = b;

(Make sure you add the blank space at the beginning of the first
line, either with several spaces or by pressing the Tab key.)
You will notice that when you press Enter after typing the first
line, the cursor goes to the next line and automatically indents
the same distance as the previous line.

The "autoindent" option automatically indents each line the same
distance as the previous one.  This can save quite a bit of work
getting the indentation right, especially when you have several
levels of indentation.

When you are entering code with "autoindent" in effect, pressing
Ctrl-T gives you another level of indentation, and pressing Ctrl-D
takes one away.  For example, copy (append) the next four lines
into the blank line that follows them.

To indent the next line, press Ctrl-T at the start of the line.
    The next line will be indented this same distance.
    On the next line, press Ctrl-D at the start of the line.
Notice how Ctrl-D moves the cursor back one level of indent.

I should point out that you press Ctrl-T and Ctrl-D while you are
in Insert Mode.

The amount of indentation provided by Ctrl-T is one
"shiftwidth".  Like tab stops, shiftwidths are set every eight
spaces by default.  A command like

:set shiftwidth=5

can change the shiftwidth for a "vi" session.

Try using the "autoindent" option when you are entering source
code.  It makes correct indentation easier.  It can even help
you avoid bugs -- correct indentation often makes it easier to
notice certain types of syntax errors.


Shift Commands:

The "<<" and ">>" commands can also help you indent source code.
By default, ">>" shifts a line right one shiftwidth; "<<" shifts
a line left one shiftwidth.  For example, move the cursor to the
beginning of this line and press ">" twice.  You will see the
line move right.  If you now press "<" twice, the line will move
back again.

You can shift several lines by typing the number followed by
">>" or "<<".  For example, move the cursor to the first line of
this paragraph and type "5>>".  You will shift all five lines in
the paragraph.  What command will move the paragraph back?  Try
it to make sure.


***************************************************************
*******        Part II: Special Search Commands         *******
***************************************************************

The characters "(", "[", "{", and "<" can all be called opening
brackets.  When the cursor is resting on one of these
characters, pressing "%" moves the cursor from the opening
bracket forward to the corresponding closing bracket -- ")",
"]", "}", or ">", keeping in mind the usual rules for nesting
brackets.  For example, move the cursor to the first "(" in

if ( cos(a[i]) > sin(b[i]+c[i]) )
{
    printf("cos and sin equal!");
}

and press "%".  You will see that the cursor jumps to the
parenthesis at the end of the line.  This is the closing
parenthesis that matches the opening one.

Similarly, if the cursor is on one of the closing bracket
characters, pressing "%" moves the cursor backward to the
corresponding opening bracket character.  For example, move the
cursor to the closing brace after the preceding "printf" line
and press "%".

Not only does "%" help you move forward and backward through a
program in long jumps, it lets you check the nesting of
parentheses in source code.  For example, if you put the cursor
on the first "{" at the beginning of a C function, pressing "%"
should move you to the "}" that (you think) ends the function.
If it doesn't, you've made a typing mistake somewhere.


***************************************************************
*******          Part III: Large-Scale Changes          *******
***************************************************************

If the name of a data object or function has to be changed in a
program for some reason, it becomes necessary to change every
occurrence of that name.  This would be a tedious process using
the "vi" features I have discussed up to this point, because you
would have to search through each source file for the name and
then type the new name wherever the old one was found.  To
avoid much of this work, "vi" offers the substitute command.


The substitute Command:

The usual form of the substitute command is

:s/pattern/replacement/

where pattern is a regular expression and replacement is any
string.  For example, move the rest of this paragraph onto the
screen, then move the cursor to THIS line and type

:s/THIS/this/

As soon as you type the colon, you see the cursor move to the
bottom of the screen.  Type the rest of the command and press
Enter.  You will see "THIS" turn into "this".  The command puts
the given replacement string in the place of the first string
that matches the given pattern.

What happens if a line has more than one string that matches
the pattern?  This line contains the word "the" several times.
If you move the cursor to the line and type

:s/the/XXX/

what happens?  "vi" only replaces the first matching string.
The position of the cursor in the line does not matter.

If you want to change every occurrence of a string on a line,
type a "g" (standing for global) after the last slash.  For
example, move the cursor to this line and type

:s/h/H/g

You will see both "h" characters change.  Notice that "global"
only refers to a single line in this context, not to the whole
file.

Now how does the "s" command help you make large-scale changes?
In addition to applying "s" to a single line, you can apply it
to a range of lines.  For example, type the following command
and press Enter:

:1,175s/^/!/

What happens?  The "1,175" in front of the "s" indicates that
the command should be applied to the lines from 1 through 175
(everything up to the 175th line in the file).  The "s" command
itself says that the beginning of the line "^" should be
replaced by an exclamation mark.  So (as you have probably seen
by now), "vi" puts an exclamation mark at the beginning of every
line up to number 175.  To get rid of the exclamation marks,
type

:1,175s/^!//

(which says, "Change every '!' at the beginning of a
line into nothing").


Line Numbers:

In the preceding instructions, I made use of line numbers to
refer to lines.  How do you know what number a line has?  If you
just want to know the number of one line, move the cursor to
that line and press Ctrl-G. For example, find out what the line
number of this line is.  For another approach, type

:set number

and press Enter.  As you can now see, this displays the number
of every line in the file.  If you want the numbers to go away,
type

:set nonumber

It's up to you whether or not you want to keep the numbers.


Line Ranges:

There are several special symbols that can be used when
specifying a range of lines.  The "." stands for the line that
currently holds the cursor.  For example, move the cursor to
this line and type

:1,.s/$/???/

This adds "???" to the end of every line from the start of the
file to the line containing the cursor.  Move the cursor down to
this line and type

:1,.s/???$//

This removes the question marks.  Notice that the command
worked, even though the last few lines before the cursor did not
end in "???".  When you issue a "substitute" command with a
range, it is all right if some of the lines in the range do not
contain the pattern you are replacing.

When specifying a range of lines, "$" stands for the last line
in the file.  Try this command

:1,$s/the/THE/g

What does it do?  It changes every "the" in the file to
uppercase (including words like "there", where "the" is part of
the word).  Now, just to put things back the way they were, type
"u" to undo the change you just made.  This demonstrates that
the "u" command can undo changes made by ":s" as well as those
made by "c" and "d".

It should be obvious now how to make a global change in a file.
For example, here's a small piece of program source code to
change.

int bubble()
{
    extern int array[30];
    int i, j, temp;
    for (i = 0; i < 30; i++)
        for (j = 29; j > i; j--)
            if (array[j] < array[j-1])
            {
                temp = array[j];
                array[j] = array[j-1];
                array[j-1] = temp;
            }
    return array[0];
}

Now, use an "s" command to change every occurrence of "array"
into "list".  Make sure you get the line range correctly; use
Ctrl-G to find out the line numbers of the first and last lines
if you no longer have the line numbers shown on your screen.
You can also use

:set number

to make the line numbers reappear on your screen.

Now check the sample source code to make sure that you changed
every occurrence of "array" into "list".  The command you should
have used to do this was

:249,262s/array/list/g

If you forgot the "g" on the end, the lines that contain "array"
twice will still have references to "array".  Without the "g",
the "s" command only changes the first occurrence of the given
pattern.


Confirmation of Substitution:

What would you do if you wanted to change the variable "i" into
a "k"?  You can't just use an instruction like

:249,262s/i/k/g

because that changes the letter "i" into "k", even when the
letter appears in words like "int" and "list".  The solution is
to add a "c" (for check) on the end of the "s" command.  When
you do this, "vi" checks with you before making every
substitution.  Before each possible change, "vi" prints the line
at the bottom of your screen and puts a "^" under the string
that is eligible to be changed.  If you want the change to
happen, press "y" followed by Enter.  If you do not want the
change to happen, press "n" followed by Enter.  To see how this
works, try

:249,262s/i/k/gc

and only change those occurrences of "i" that are related to the
variable "i".

Using ":s" is much faster (and less error-prone) than trying to
change every occurrence of a symbol by hand.  Remember the "c"
option if you think there's a chance that the name you are
trying to change might be part of a larger symbol.


***************************************************************
*******           Part IV: Executing Commands           *******
***************************************************************

Suppose you have just finished creating a source code file using
"vi" and you are ready to compile it.  You can write out the
file, quit "vi", and then compile it if you want...but everybody
knows that there are likely to be errors the first time you
compile a new file.  To fix these compilation errors, you'll
have to use "vi" again to edit the source code.

To avoid the time and effort of quitting "vi" to compile, then
going back into "vi" to fix errors, you can invoke the compile
command directly from "vi".  A step-by-step description of how
you can do this follows; however, you can't do it right now
because you're looking at this exercise file, and not at a
program's source code.

(a)  First, write out your source code to a file by saying

:w filename

You don't have to specify the filename if you've already
written to the file once before, or if you originally read the
text in from the file.

(b)  Type a colon (:), followed by an exclamation mark (!),
followed by the command you would normally use to compile that
file.  On a true UNIX system, this might be

:!cc filename

because "cc" is the usual command for compiling C code under
UNIX. This won't work on MKS Tools, because the software doesn't
include a C compiler.  Press Enter when you have typed the line.
"cc" compiles the program, and displays any diagnostic messages
in the usual way.

(c)  After the compilation is finished, you automatically return
to "vi", where you can correct any errors that might have been
detected.

The ability to execute commands from inside "vi" has other
benefits.  For example, enter

:!more sonnet

This shows that you can use "more" if you want to look up
information in some other file.  If you were really programming,
you might want to check another source code file for the name of
a function or variable.
